@@ -161,8 +161,8 @@ def run(ns):
     # INITIALIZATION #
     ##################
 
-    # scg.set_MDA_backend(ns)  # cannot use this properly currently because we cannot limit number of threads
-    ns.mda_backend = 'serial'  # currently force single thread for safe clusters execution
+    # scg.set_MDA_backend(ns)
+    ns.mda_backend = 'serial'  # actually serial is faster because MDA is not properly parallelized atm
 
     # directory to write all files for current execution of optimizations routines
     os.mkdir(ns.exec_folder)
@@ -204,6 +204,7 @@ def run(ns):
     scg.get_atoms_weights_in_beads(ns)  # get weights of atoms within beads
 
     scg.read_cg_itp_file(ns)  # load the ITP object and find out geoms grouping
+    ns.cg_itp_input = copy.deepcopy(ns.cg_itp)
     scg.process_scaling_str(ns)  # process the bonds scaling specified by user
 
     print()
@@ -272,19 +273,20 @@ def run(ns):
     for grp_constraint in range(ns.nb_constraints):
 
         constraint_avg, constraint_hist, constraint_values = scg.get_AA_bonds_distrib(ns, beads_ids=ns.cg_itp['constraint'][grp_constraint]['beads'], grp_type='constraint group', grp_nb=grp_constraint)
-        # if ns.exec_mode == 1:
-        ns.cg_itp['constraint'][grp_constraint]['value'] = constraint_avg
+        if ns.exec_mode == 1:
+            ns.cg_itp['constraint'][grp_constraint]['value'] = constraint_avg
         ns.cg_itp['constraint'][grp_constraint]['avg'] = constraint_avg
         ns.cg_itp['constraint'][grp_constraint]['hist'] = constraint_hist
 
         ns.domains_val['constraint'].append([round(np.min(constraint_values), 3), round(np.max(constraint_values), 3)])
+        print(f'  Constraint grp {grp_constraint+1} -- Average value: '+str(round(constraint_avg, 2))+' nm -- Initial equilibrium value: '+str(ns.cg_itp['constraint'][grp_constraint]['value'])+' nm')
 
     # get ref atom hists + find very first distances and force constants guesses for bonds groups
     for grp_bond in range(ns.nb_bonds):
 
         bond_avg, bond_hist, bond_values = scg.get_AA_bonds_distrib(ns, beads_ids=ns.cg_itp['bond'][grp_bond]['beads'], grp_type='bond group', grp_nb=grp_bond)
-        # if ns.exec_mode == 1:
-        ns.cg_itp['bond'][grp_bond]['value'] = bond_avg
+        if ns.exec_mode == 1:
+            ns.cg_itp['bond'][grp_bond]['value'] = bond_avg
         ns.cg_itp['bond'][grp_bond]['avg'] = bond_avg
         ns.cg_itp['bond'][grp_bond]['hist'] = bond_hist
 
@@ -293,12 +295,13 @@ def run(ns):
         ns.data_BI['bond'].append([np.histogram(bond_values, range=(xmin, xmax), bins=config.bi_nb_bins)[0], np.std(bond_values), np.mean(bond_values), (xmin, xmax)])
 
         ns.domains_val['bond'].append([round(np.min(bond_values), 3), round(np.max(bond_values), 3)])  # boundaries of force constats during optimization
+        print(f'  Bond grp {grp_bond+1} -- Average value: '+str(round(bond_avg, 2))+' nm -- Initial equilibrium value: '+str(ns.cg_itp['bond'][grp_bond]['value'])+' nm')
 
     # get ref atom hists + find very first values and force constants guesses for angles groups
     for grp_angle in range(ns.nb_angles):
 
         angle_avg, angle_hist, angle_values_deg, angle_values_rad = scg.get_AA_angles_distrib(ns, beads_ids=ns.cg_itp['angle'][grp_angle]['beads'])
-        if ns.exec_mode == 1 or ns.exec_mode == 3:
+        if ns.exec_mode == 1:
             ns.cg_itp['angle'][grp_angle]['value'] = angle_avg
         ns.cg_itp['angle'][grp_angle]['avg'] = angle_avg
         ns.cg_itp['angle'][grp_angle]['hist'] = angle_hist
@@ -308,12 +311,13 @@ def run(ns):
         ns.data_BI['angle'].append([np.histogram(angle_values_rad, range=(np.deg2rad(xmin), np.deg2rad(xmax)), bins=config.bi_nb_bins)[0], np.std(angle_values_rad), (xmin, xmax)])
 
         ns.domains_val['angle'].append([round(np.min(angle_values_deg), 2), round(np.max(angle_values_deg), 2)])  # boundaries of force constants during optimization
+        print(f'  Angle grp {grp_angle+1} -- Average value: ' + str(round(angle_avg, 2)) + ' degrees -- Initial equilibrium value: '+str(ns.cg_itp['angle'][grp_angle]['value'])+' degrees')
 
     # get ref atom hists + find very first values and force constants guesses for dihedrals groups
     for grp_dihedral in range(ns.nb_dihedrals):
 
         dihedral_avg, dihedral_hist, dihedral_values_deg, dihedral_values_rad = scg.get_AA_dihedrals_distrib(ns, beads_ids=ns.cg_itp['dihedral'][grp_dihedral]['beads'])
-        if ns.exec_mode == 1:  # the angle value for dihedral will be calculated from the BI fit, because for dihedrals it makes no sense to use the average
+        if ns.exec_mode == 1:  # the dihedral equi value will be calculated from the BI fit, because for dihedrals it makes no sense to use the average
             ns.cg_itp['dihedral'][grp_dihedral]['value'] = dihedral_avg
         ns.cg_itp['dihedral'][grp_dihedral]['avg'] = dihedral_avg
         ns.cg_itp['dihedral'][grp_dihedral]['hist'] = dihedral_hist
@@ -322,6 +326,7 @@ def run(ns):
         ns.data_BI['dihedral'].append([np.histogram(dihedral_values_rad, range=(np.deg2rad(xmin), np.deg2rad(xmax)), bins=2 *config.bi_nb_bins)[0], np.std(dihedral_values_rad), np.mean(dihedral_values_rad), (xmin, xmax)])
 
         ns.domains_val['dihedral'].append([round(np.min(dihedral_values_deg), 2), round(np.max(dihedral_values_deg), 2)])  # boundaries of force constats during optimization
+        print(f'  Dihedral grp {grp_dihedral+1} -- Average value: ' + str(round(dihedral_avg, 2)) + ' degrees -- Initial equilibrium value: '+str(ns.cg_itp['dihedral'][grp_dihedral]['value'])+' degrees')
 
     if not ns.bonds_rescaling_performed:
         print('  No bonds rescaling performed')
@@ -488,8 +493,7 @@ def run(ns):
         # actual BI to get the initial guesses of force constants, for all selected geoms at this given optimization step
         # BI is performed:
         # -- exec_mode 1: all equilibrium values and force constants
-        # -- exec_mode 2: equilibrium values are not touched for angles and dihedrals, but all force constants are estimated
-        # -- exec_mode 3: equilibrium values are not touched for dihedrals, but all force constants are estimated
+        # -- exec_mode 2: equilibrium values are not touched for bonds, angles and dihedrals, but all their force constants are optimized
         scg.perform_BI(ns)  # performed on object ns.out_itp
 
         # build vector for search space boundaries + create variations around the BI initial guesses
@@ -567,7 +571,7 @@ def main():
     optional_args0 = args_parser.add_argument_group(
     req_args_header + '\n\n' + bullet + 'EXECUTION MODE')
     optional_args0.add_argument('-exec_mode', dest='exec_mode',
-                              help='MODE 1: Tune both bonds lengths, angles/dihedrals values\n        and their force constants\nMODE 2: Like MODE 1 but angles/dihedrals values in the prelim.\n        CG model ITP are conserved during optimization\nMODE 3: Like MODE 1 but only dihedrals values in the prelim.\n        CG model ITP are conserved during optimization',
+                              help='MODE 1: Tune both bonds/angles/dihedrals equilibrium values\n        and their force constants\nMODE 2: Tune only bonds/angles/dihedrals force constants\n        with FIXED equilibrium values from the prelim. CG ITP',
                               type=int, default=1, metavar='              (1)')
 
     required_args = args_parser.add_argument_group(bullet + 'REFERENCE AA MODEL')
@@ -588,6 +592,9 @@ def main():
                                   help='ITP file of the CG model to optimize', type=str,
                                   default=config.metavar_cg_itp,
                                   metavar='      ' + scg.par_wrap(config.metavar_cg_itp))
+    sim_filenames_args.add_argument('-user_params', dest='user_input',
+                                help='If absent, only the BI is used as starting point for parametrization\nIf present, parameters in the input ITP files are considered\n',
+                                action='store_true', default=False)
     sim_filenames_args.add_argument('-cg_gro', dest='gro_input_filename',
                                   help='Starting GRO file used for iterative simulation\nWill be minimized and relaxed before each MD run',
                                   type=str, default='start_conf.gro',
@@ -728,6 +735,10 @@ def main():
     # arguments handling, display command line if help or no arguments provided
     # argcomplete.autocomplete(parser)
     ns = args_parser.parse_args()
+
+    if not ns.verbose:
+        sys.tracebacklimit = 0
+
     input_cmdline = ' '.join(map(cmd_quote, sys.argv))
     ns.exec_folder = time.strftime(
     "MODEL_OPTI__STARTED_%d-%m-%Y_%Hh%Mm%Ss")  # default folder name for all files of this optimization run, in case none is provided
